/*
 * Copyright (C) 2018 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * Author: Droiing <jianglinxuan@kylinos.cn>
 *         chenziyi <chenziyi@kylinos.cn>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#include <stdbool.h>
#include <stdio.h>
#include <string.h>

#include "aes_128_cfb.h"

static unsigned char SBox[] = {
	/*0     1     2     3     4     5     6     7     8     9     a     b     c     d     e     f */
	0x63, 0x7c, 0x77, 0x7b, 0xf2, 0x6b, 0x6f, 0xc5, 0x30, 0x01, 0x67, 0x2b, 0xfe, 0xd7, 0xab, 0x76, /*0*/
	0xca, 0x82, 0xc9, 0x7d, 0xfa, 0x59, 0x47, 0xf0, 0xad, 0xd4, 0xa2, 0xaf, 0x9c, 0xa4, 0x72, 0xc0, /*1*/
	0xb7, 0xfd, 0x93, 0x26, 0x36, 0x3f, 0xf7, 0xcc, 0x34, 0xa5, 0xe5, 0xf1, 0x71, 0xd8, 0x31, 0x15, /*2*/
	0x04, 0xc7, 0x23, 0xc3, 0x18, 0x96, 0x05, 0x9a, 0x07, 0x12, 0x80, 0xe2, 0xeb, 0x27, 0xb2, 0x75, /*3*/
	0x09, 0x83, 0x2c, 0x1a, 0x1b, 0x6e, 0x5a, 0xa0, 0x52, 0x3b, 0xd6, 0xb3, 0x29, 0xe3, 0x2f, 0x84, /*4*/
	0x53, 0xd1, 0x00, 0xed, 0x20, 0xfc, 0xb1, 0x5b, 0x6a, 0xcb, 0xbe, 0x39, 0x4a, 0x4c, 0x58, 0xcf, /*5*/
	0xd0, 0xef, 0xaa, 0xfb, 0x43, 0x4d, 0x33, 0x85, 0x45, 0xf9, 0x02, 0x7f, 0x50, 0x3c, 0x9f, 0xa8, /*6*/
	0x51, 0xa3, 0x40, 0x8f, 0x92, 0x9d, 0x38, 0xf5, 0xbc, 0xb6, 0xda, 0x21, 0x10, 0xff, 0xf3, 0xd2, /*7*/
	0xcd, 0x0c, 0x13, 0xec, 0x5f, 0x97, 0x44, 0x17, 0xc4, 0xa7, 0x7e, 0x3d, 0x64, 0x5d, 0x19, 0x73, /*8*/
	0x60, 0x81, 0x4f, 0xdc, 0x22, 0x2a, 0x90, 0x88, 0x46, 0xee, 0xb8, 0x14, 0xde, 0x5e, 0x0b, 0xdb, /*9*/
	0xe0, 0x32, 0x3a, 0x0a, 0x49, 0x06, 0x24, 0x5c, 0xc2, 0xd3, 0xac, 0x62, 0x91, 0x95, 0xe4, 0x79, /*a*/
	0xe7, 0xc8, 0x37, 0x6d, 0x8d, 0xd5, 0x4e, 0xa9, 0x6c, 0x56, 0xf4, 0xea, 0x65, 0x7a, 0xae, 0x08, /*b*/
	0xba, 0x78, 0x25, 0x2e, 0x1c, 0xa6, 0xb4, 0xc6, 0xe8, 0xdd, 0x74, 0x1f, 0x4b, 0xbd, 0x8b, 0x8a, /*c*/
	0x70, 0x3e, 0xb5, 0x66, 0x48, 0x03, 0xf6, 0x0e, 0x61, 0x35, 0x57, 0xb9, 0x86, 0xc1, 0x1d, 0x9e, /*d*/
	0xe1, 0xf8, 0x98, 0x11, 0x69, 0xd9, 0x8e, 0x94, 0x9b, 0x1e, 0x87, 0xe9, 0xce, 0x55, 0x28, 0xdf, /*e*/
	0x8c, 0xa1, 0x89, 0x0d, 0xbf, 0xe6, 0x42, 0x68, 0x41, 0x99, 0x2d, 0x0f, 0xb0, 0x54, 0xbb, 0x16 /*f*/
};
static unsigned char InvSBox[] = {
	/*0     1     2     3     4     5     6     7     8     9     a     b     c     d     e     f */
	0x52, 0x09, 0x6a, 0xd5, 0x30, 0x36, 0xa5, 0x38, 0xbf, 0x40, 0xa3, 0x9e, 0x81, 0xf3, 0xd7, 0xfb, /*0*/
	0x7c, 0xe3, 0x39, 0x82, 0x9b, 0x2f, 0xff, 0x87, 0x34, 0x8e, 0x43, 0x44, 0xc4, 0xde, 0xe9, 0xcb, /*1*/
	0x54, 0x7b, 0x94, 0x32, 0xa6, 0xc2, 0x23, 0x3d, 0xee, 0x4c, 0x95, 0x0b, 0x42, 0xfa, 0xc3, 0x4e, /*2*/
	0x08, 0x2e, 0xa1, 0x66, 0x28, 0xd9, 0x24, 0xb2, 0x76, 0x5b, 0xa2, 0x49, 0x6d, 0x8b, 0xd1, 0x25, /*3*/
	0x72, 0xf8, 0xf6, 0x64, 0x86, 0x68, 0x98, 0x16, 0xd4, 0xa4, 0x5c, 0xcc, 0x5d, 0x65, 0xb6, 0x92, /*4*/
	0x6c, 0x70, 0x48, 0x50, 0xfd, 0xed, 0xb9, 0xda, 0x5e, 0x15, 0x46, 0x57, 0xa7, 0x8d, 0x9d, 0x84, /*5*/
	0x90, 0xd8, 0xab, 0x00, 0x8c, 0xbc, 0xd3, 0x0a, 0xf7, 0xe4, 0x58, 0x05, 0xb8, 0xb3, 0x45, 0x06, /*6*/
	0xd0, 0x2c, 0x1e, 0x8f, 0xca, 0x3f, 0x0f, 0x02, 0xc1, 0xaf, 0xbd, 0x03, 0x01, 0x13, 0x8a, 0x6b, /*7*/
	0x3a, 0x91, 0x11, 0x41, 0x4f, 0x67, 0xdc, 0xea, 0x97, 0xf2, 0xcf, 0xce, 0xf0, 0xb4, 0xe6, 0x73, /*8*/
	0x96, 0xac, 0x74, 0x22, 0xe7, 0xad, 0x35, 0x85, 0xe2, 0xf9, 0x37, 0xe8, 0x1c, 0x75, 0xdf, 0x6e, /*9*/
	0x47, 0xf1, 0x1a, 0x71, 0x1d, 0x29, 0xc5, 0x89, 0x6f, 0xb7, 0x62, 0x0e, 0xaa, 0x18, 0xbe, 0x1b, /*a*/
	0xfc, 0x56, 0x3e, 0x4b, 0xc6, 0xd2, 0x79, 0x20, 0x9a, 0xdb, 0xc0, 0xfe, 0x78, 0xcd, 0x5a, 0xf4, /*b*/
	0x1f, 0xdd, 0xa8, 0x33, 0x88, 0x07, 0xc7, 0x31, 0xb1, 0x12, 0x10, 0x59, 0x27, 0x80, 0xec, 0x5f, /*c*/
	0x60, 0x51, 0x7f, 0xa9, 0x19, 0xb5, 0x4a, 0x0d, 0x2d, 0xe5, 0x7a, 0x9f, 0x93, 0xc9, 0x9c, 0xef, /*d*/
	0xa0, 0xe0, 0x3b, 0x4d, 0xae, 0x2a, 0xf5, 0xb0, 0xc8, 0xeb, 0xbb, 0x3c, 0x83, 0x53, 0x99, 0x61, /*e*/
	0x17, 0x2b, 0x04, 0x7e, 0xba, 0x77, 0xd6, 0x26, 0xe1, 0x69, 0x14, 0x63, 0x55, 0x21, 0x0c, 0x7d /*f*/
};

void keyExpansion(unsigned char* key, unsigned char w[][4][4])
{
	int i, j, r, c;
	unsigned char rc[] = {0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x1b, 0x36};

	for (r = 0; r < 4; r++)
		for (c = 0; c < 4; c++)
			w[0][r][c] = key[r + c * 4];

	for (i = 1; i <= 10; i++)
		for (j = 0; j < 4; j++) {
			unsigned char t[4];

			for (r = 0; r < 4; r++)
				t[r] = j ? w[i][r][j - 1] : w[i - 1][r][3];

			if (j == 0) {
				unsigned char temp = t[0];
				for (r = 0; r < 3; r++) {
					t[r] = SBox[t[(r + 1) % 4]];
				}
				t[3] = SBox[temp];
				t[0] ^= rc[i - 1];
			}

			for (r = 0; r < 4; r++)
				w[i][r][j] = w[i - 1][r][j] ^ t[r];
		}
}

unsigned char* cipher(unsigned char* input, unsigned char* output, unsigned char w[][4][4])
{
	unsigned char state[4][4];
	int i, r, c;

	for (r = 0; r < 4; r++)
		for (c = 0; c < 4; c++)
			state[r][c] = input[c * 4 + r];

	addRoundKey(state, w[0]);

	for (i = 1; i <= 10; i++) {
		subBytes(state);
		shiftRows(state);
		if (i != 10)
			mixColumns(state);
		addRoundKey(state, w[i]);
	}

	for (r = 0; r < 4; r++)
		for (c = 0; c < 4; c++)
			output[c * 4 + r] = state[r][c];

	return output;
}

unsigned char* invCipher(unsigned char* input, unsigned char* output, unsigned char w[][4][4])
{
	unsigned char state[4][4];
	int i, r, c;

	for (r = 0; r < 4; r++)
		for (c = 0; c < 4; c++)
			state[r][c] = input[c * 4 + r];

	addRoundKey(state, w[10]);
	for (i = 9; i >= 0; i--) {
		invShiftRows(state);
		invSubBytes(state);
		addRoundKey(state, w[i]);
		if (i)
			invMixColumns(state);
	}

	for (r = 0; r < 4; r++)
		for (c = 0; c < 4; c++)
			output[c * 4 + r] = state[r][c];
	return (output);
}

unsigned char FFmul(unsigned char a, unsigned char b)
{
	unsigned char bw[4];
	unsigned char res = 0;
	int i;

	bw[0] = b;
	for (i = 1; i < 4; i++) {
		bw[i] = bw[i - 1] << 1;
		if (bw[i - 1] & 0x80)
			bw[i] ^= 0x1b;
	}

	for (i = 0; i < 4; i++)
		if ((a >> i) & 0x01)
			res ^= bw[i];

	return res;
}

void subBytes(unsigned char state[][4])
{
	int r, c;

	for (r = 0; r < 4; r++)
		for (c = 0; c < 4; c++)
			state[r][c] = SBox[state[r][c]];
}

void shiftRows(unsigned char state[][4])
{
	unsigned char t[4];
	int r, c;

	for (r = 1; r < 4; r++) {
		for (c = 0; c < 4; c++)
			t[c] = state[r][(c + r) % 4];

	for (c = 0; c < 4; c++)
		state[r][c] = t[c];
	}
}

void mixColumns(unsigned char state[][4])
{
	unsigned char t[4];
	int r, c;

	for (c = 0; c < 4; c++) {
		for (r = 0; r < 4; r++)
			t[r] = state[r][c];

		for (r = 0; r < 4; r++)
			state[r][c] = FFmul(0x02, t[r]) ^
				      FFmul(0x03, t[(r + 1) % 4]) ^
				      FFmul(0x01, t[(r + 2) % 4]) ^
				      FFmul(0x01, t[(r + 3) % 4]);
	}
}

void invSubBytes(unsigned char state[][4])
{
	int r, c;

	for (r = 0; r < 4; r++)
		for (c = 0; c < 4; c++)
			state[r][c] = InvSBox[state[r][c]];
}

void invShiftRows(unsigned char state[][4])
{
	unsigned char t[4];
	int r, c;

	for (r = 1; r < 4; r++) {
		for (c = 0; c < 4; c++)
			t[c] = state[r][(c - r + 4) % 4];

	for (c = 0; c < 4; c++)
		state[r][c] = t[c];
	}
}

void invMixColumns(unsigned char state[][4])
{
	unsigned char t[4];
	int r, c;

	for (c = 0; c < 4; c++) {
		for (r = 0; r < 4; r++)
			t[r] = state[r][c];

		for (r = 0; r < 4; r++)
			state[r][c] = FFmul(0x0e, t[r]) ^
				      FFmul(0x0b, t[(r + 1) % 4]) ^
				      FFmul(0x0d, t[(r + 2) % 4]) ^
				      FFmul(0x09, t[(r + 3) % 4]);
	}
}

void addRoundKey(unsigned char state[][4], unsigned char k[][4])
{
	int r, c;

	for (c = 0; c < 4; c++)
		for (r = 0; r < 4; r++)
			state[r][c] ^= k[r][c];
}

/**************************************************************/

int AES_128_CFB_Encrypt(unsigned char* key, unsigned char* iv,
			unsigned char* inData, int inLen, unsigned char* outData)
{
	bool firstRound = true;
	int r = 0;
	int rounds = 0;
	int start = 0;
	int end = 0;
	int outLen = 0;

	unsigned char input[16] = {0};
	unsigned char output[16] = {0};
	unsigned char plainText[16] = {0};
	unsigned char cipherText[16] = {0};
	static unsigned char w[11][4][4] = {0};

	memset(outData, 0, inLen);
	keyExpansion(key, w);

	if (inLen % 16 == 0)
		rounds = inLen / 16;
	else
		rounds = inLen / 16 + 1;

	for (r = 0; r < rounds; r++) {
		start = r * 16;
		end = r * 16 + 16;
		if (end > inLen)
			end = inLen;

		memset(plainText, 0, 16);
		memcpy(plainText, inData + start, end - start);

		if (firstRound == true) {
			cipher(iv, output, w);
			firstRound = false;
		}
		else
			cipher(input, output, w);

		int i;
		for (i = 0; i < 16; i++)
			if ((end - start) - 1 < i)
				cipherText[i] = 0 ^ output[i];
			else
				cipherText[i] = plainText[i] ^ output[i];

		for (i = 0; i < end - start; i++)
			outData[outLen++] = cipherText[i];

		memcpy(input, cipherText, 16);
	}

	return outLen;
}

int AES_128_CFB_Decrypt(unsigned char* key, unsigned char* iv,
			unsigned char* inData, int inLen, unsigned char* outData)
{
	bool firstRound = true;
	int r = 0;
	int rounds = 0;
	int start = 0;
	int end = 0;
	int outLen = 0;

	unsigned char input[16] = {0};
	unsigned char output[16] = {0};
	unsigned char cipherText[16] = {0};
	unsigned char plainText[16] = {0};
	static unsigned char w[11][4][4] = {0};

	memset(outData, 0, inLen);
	keyExpansion(key, w);

	if (inLen % 16 == 0)
		rounds = inLen / 16;
	else
		rounds = inLen / 16 + 1;

	for (r = 0; r < rounds; r++) {
		start = r * 16;
		end = start + 16;
		if (end > inLen)
			end = inLen;

		memset(cipherText, 0, 16);
		memcpy(cipherText, inData + start, end - start);

		if (firstRound == true) {
			cipher(iv, output, w);
			firstRound = false;
		}
		else
			cipher(input, output, w);

		int i;
		for (i = 0; i < 16; i++)
			if (end - start - 1 < i)
				plainText[i] = output[i] ^ 0;
			else
				plainText[i] = output[i] ^ cipherText[i];

		for (i = 0; i < end - start; i++)
			outData[outLen++] = plainText[i];

		memcpy(input, cipherText, 16);
	}

	return outLen;
}
